from typing import List, TextIO


def parse_ints(io: TextIO) -> List[int]:
    """Parse first line of io to list of integers.

    Parameters
    ----------
    io :: TextIO
        Object supporting `readline()`

    Returns
    -------
    integers :: List[int]
        A list of integers

    Examples
    --------
    >>> data = StringIO("1 2 3 4")
    >>> parse_ints(data)
    [1, 2, 3, 4]
    """
    line = io.readline()
    line = line.strip()
    parts = line.split(" ")
    ints = map(int, parts)
    return list(ints)


def parse_floats(io: TextIO) -> List[float]:
    """Parse first line of io to list of floats.

    Parameters
    ----------
    io :: TextIO
        Object supporting `readline()`

    Returns
    -------
    floats :: List[float]
        A list of floats

    Examples
    --------
    >>> data = StringIO("1.1 2.2 3.3 4.4")
    >>> parse_floats(data)
    [1.1, 2.2, 3.3, 4.4]
    """
    line = io.readline()
    line = line.strip()
    parts = line.split(" ")
    ints = map(float, parts)
    return list(ints)


def get_triangles(mesh):
    """ Return tuple (X, Y, T) of triangular data.

    Data can be used effectively in matplotlib's `triplot`:

    >>> X, Y, T = get_triangles(mesh)
    >>> plt.triplot(X, Y, T)
    """
    elements = {}
    nodes = {}
    node_ids = set()

    for entity in mesh.get_element_entities():
        eltype = entity.get_element_type()
        if entity.get_dimension() == 2 and eltype == 2:
            for element in entity.get_elements():
                elid = element.get_tag()
                elcon = element.get_connectivity()
                elements[elid] = elcon
                for c in elcon:
                    node_ids.add(c)

    for entity in mesh.get_node_entities():
        for node in entity.get_nodes():
            nid = node.get_tag()
            if nid not in node_ids:
                continue
            ncoords = node.get_coordinates()
            nodes[nid] = ncoords

    invP = {}
    X = []
    Y = []

    for (i, nid) in enumerate(node_ids):
        invP[nid] = i
        X.append(nodes[nid][0])
        Y.append(nodes[nid][1])

    T = []
    for element in elements.values():
        T.append([invP[c] for c in element])

    return X, Y, T



import setuptools

# https://packaging.python.org/guides/single-sourcing-package-version/

import codecs
import os.path


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


with open("README.md", "r") as fh:
    long_description = fh.read()

test_deps = [
    'pytest',
    'pytest-cov',
    'pytest-pycodestyle',
    'pytest-pep8',
    'pytest-flake8',
]

extras = {
    'test': test_deps,
}

description = """
gmshparser is a lightweight, 100 % tested and well documented package that aims
to reliably parse the Gmsh ascii file format (.msh). The package does not
introduce any external dependencies and thus fits well with the needs of your
own FEM research code as a small stand-alone package. Project is hosted on
GitHub (https://github.com/ahojukka5/gmshparser) and documentation can be
found from ReadTheDocs (https://gmshparser.readthedocs.io/).
"""

entry_points = {
    'console_scripts': ['gmshparser=gmshparser.cli:main'],
}

setuptools.setup(
    name="gmshparser",
    version=get_version("gmshparser/__init__.py"),
    author="Jukka Aho",
    author_email="ahojukka5@gmail.com",
    description=" ".join(description.strip().splitlines()),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ahojukka5/gmshparser",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    tests_require=test_deps,
    extras_require=extras,
    entry_points=entry_points,
)
